#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_cpu_schema[]=
{
  {"CPU_ID", SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, MY_I_S_UNSIGNED, 
             "Cpu Id"},
  {"VENDOR", 128,                 MYSQL_TYPE_VARCHAR,  0, 0, "Cpu_vendor"},
  {"MODEL",  128,                 MYSQL_TYPE_VARCHAR,  0, 0, "Cpu_model"},
  {"MHZ",    SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, 0, "Cpu_mhz"},
  {"CACHE",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
             MY_I_S_UNSIGNED, "Cpu_cache"},
  {"USER",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
             MY_I_S_UNSIGNED, "Cpu_user"},
  {"SYS",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
             MY_I_S_UNSIGNED, "Cpu_sys"},
  {"NICE",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
             MY_I_S_UNSIGNED, "Cpu_nice"},
  {"IDLE",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
             MY_I_S_UNSIGNED, "Cpu_idle"},
  {"WAIT",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
             MY_I_S_UNSIGNED, "Cpu_wait"},
  {"TOTAL",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
             MY_I_S_UNSIGNED, "Cpu_total"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};


int os_cpu_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("cpu_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each cpu */
    sigar_cpu_info_list_t cpuinfolist;
    sigar_cpu_list_t cpulist;
  
    return_code= sigar_cpu_info_list_get(t, &cpuinfolist);
    return_code= return_code | sigar_cpu_list_get(t, &cpulist);
    
    /*if all or some data fetched */
    if (return_code == SIGAR_OK && cpuinfolist.number == cpulist.number) 
    {
      for (i = 0; i < cpuinfolist.number; i++) 
      {
        sigar_cpu_info_t *cpuinfo= &cpuinfolist.data[i];
        sigar_cpu_t *cpu=          &cpulist.data[i];
        
        table->field[0]->store(i+1, SP_UNSIGNED);
        if (cpuinfo != NULL)
        {
          table->field[1]->store(cpuinfo->vendor, strlen(cpuinfo->vendor), scs);
          table->field[2]->store(cpuinfo->model, strlen(cpuinfo->model), scs);
          table->field[3]->store(cpuinfo->mhz, SP_SIGNED);
          table->field[4]->store(cpuinfo->cache_size, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_CPU: Couldn't fetch cpuinfo data for CPU%d", i);
        }
        
        if (cpu != NULL)
        {
          table->field[5]->store(cpu->user, SP_UNSIGNED);
          table->field[6]->store(cpu->sys, SP_UNSIGNED);
          table->field[7]->store(cpu->nice, SP_UNSIGNED);
          table->field[8]->store(cpu->idle, SP_UNSIGNED);
          table->field[9]->store(cpu->wait, SP_UNSIGNED);
          table->field[10]->store(cpu->total, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_CPU: Couldn't fetch cpu data for CPU%d", i);
        }
       
        error= error | schema_table_store_record(thd, table);
      }
      
      sigar_cpu_list_destroy(t, &cpulist);
      sigar_cpu_info_list_destroy(t, &cpuinfolist);
    } 
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
  
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_cpu_plugin_init(void *p)
{
  DBUG_ENTER("init_cpu_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info=     os_cpu_schema;
  schema->fill_table=      os_cpu_fill_table;
  
  DBUG_RETURN(0);
}


static int os_cpu_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_cpu_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_cpu_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_cpu= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_cpu_plugin,                                 
  "OS_CPU",                                       
  SP_AUTHOR,                 
  "SIGAR CPU Details.",                           
  PLUGIN_LICENSE_GPL,                              
  os_cpu_plugin_init,                             
  os_cpu_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
