#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_disk_schema[]=
{
  {"DEVICE_NAME",   SIGAR_FS_NAME_LEN,   
                    MYSQL_TYPE_VARCHAR,  0, 0, "Device_name"},
  {"READS",         SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Reads"},
  {"WRITES",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Writes"},
  {"BYTES_READ",    SP_ANY_INT64_LENGTH, 
                    MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, "Bytes_read"},
  {"BYTES_WRITTEN", SP_ANY_INT64_LENGTH, 
                    MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, "Bytes_written"},
  {"QUEUE",         SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Queue"},
  {"READ_TIME",     SP_ANY_INT64_LENGTH, 
                    MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, "Read_time"},
  {"WRITE_TIME",    SP_ANY_INT64_LENGTH, 
                    MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, "Write_time"},
  {"TIME",          SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Time"},
  {"SNAP_TIME",     SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Snaptime"},
  {"SERVICE_TIME",  20,                  
                    MYSQL_TYPE_DOUBLE,   0, 0, "Service_time"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};


int os_disk_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("disk_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each disk */
    sigar_file_system_list_t fslist;
    return_code= sigar_file_system_list_get(t, &fslist);
    
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < fslist.number; i++) 
      {
        sigar_file_system_t *fs = &fslist.data[i];
        sigar_disk_usage_t *diskusage;
        diskusage= (sigar_disk_usage_t*) malloc(sizeof(sigar_disk_usage_t));
        
        table->field[0]->store(fs->dev_name, strlen(fs->dev_name), scs);
        return_code= sigar_disk_usage_get(t, fs->dev_name, diskusage);
        if (return_code == SIGAR_OK)
        {
          table->field[1]->store(diskusage->reads, SP_UNSIGNED);
          table->field[2]->store(diskusage->writes, SP_UNSIGNED);
          table->field[3]->store(diskusage->read_bytes, SP_UNSIGNED);
          table->field[4]->store(diskusage->write_bytes, SP_UNSIGNED);
          table->field[5]->store(diskusage->queue, SP_UNSIGNED);
          table->field[6]->store(diskusage->rtime, SP_UNSIGNED);
          table->field[7]->store(diskusage->wtime, SP_UNSIGNED);
          table->field[8]->store(diskusage->time, SP_UNSIGNED);
          table->field[9]->store(diskusage->snaptime, SP_UNSIGNED);
          table->field[10]->store(diskusage->service_time);
        }
        else
        {
          sql_print_warning("OS_DISK_USAGE: Couldn't fetch data for %s.", 
                            fs->dev_name);  
        }
   
        error= error | schema_table_store_record(thd, table);
        free(diskusage);     
      }
  
      sigar_file_system_list_destroy(t, &fslist);    
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_disk_plugin_init(void *p)
{
  DBUG_ENTER("init_disk_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_disk_schema;
  schema->fill_table= os_disk_fill_table;
  
  DBUG_RETURN(0);
}


static int os_disk_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_disk_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_disk_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_disk= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_disk_plugin,                                 
  "OS_DISK_USAGE",                                       
  SP_AUTHOR,                 
  "SIGAR Disk Usage Information.",                           
  PLUGIN_LICENSE_GPL,                              
  os_disk_plugin_init,                             
  os_disk_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
