#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_fs_schema[]=
{
  {"DIRECTORY",   SIGAR_FS_NAME_LEN,   MYSQL_TYPE_VARCHAR,  0, 0, "Directory"},
  {"DEVICE",      SIGAR_FS_NAME_LEN,   MYSQL_TYPE_VARCHAR,  0, 0, "Device"},
  {"TYPE",        SIGAR_FS_NAME_LEN,   MYSQL_TYPE_VARCHAR,  0, 0, "Type"},
  {"SYSTEM_TYPE", SIGAR_FS_NAME_LEN,   MYSQL_TYPE_VARCHAR,  0, 0, "System_type"},
  {"FLAGS",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Flags"},
  {"USAGE",       20,                  MYSQL_TYPE_DOUBLE,   0, 0, "Usage"},
  {"TOTAL",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Total"},
  {"FREE",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Free"},
  {"USED",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Used"},
  {"AVAILABLE",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Available"},
  {"FILES",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Files"},
  {"FREE_FILES",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Free_files"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};


int os_fs_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("fs_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on file systems */
    sigar_file_system_list_t fslist;
    return_code= sigar_file_system_list_get(t, &fslist);
    
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < fslist.number; i++) 
      {
        sigar_file_system_t *fs = &fslist.data[i];
        sigar_file_system_usage_t *fsusage;
        fsusage= (sigar_file_system_usage_t*) 
                  malloc(sizeof(sigar_file_system_usage_t));
        
        if (fs != NULL)
        {
          table->field[0]->store(fs->dir_name, strlen(fs->dir_name), scs);
          table->field[1]->store(fs->dev_name, strlen(fs->dev_name), scs);
          table->field[2]->store(fs->type_name, strlen(fs->type_name), scs);
          table->field[3]->store(fs->sys_type_name, strlen(fs->sys_type_name), 
                                 scs);
          table->field[4]->store(fs->flags, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_FILESYSTEM: Couldn't fetch fs data for fslist "
                            "%d.", i);  
        }
        
        return_code= sigar_file_system_usage_get(t, fs->dir_name, fsusage);
        if (return_code == SIGAR_OK)
        {
          table->field[5]->store(fsusage->use_percent);
          table->field[6]->store(fsusage->total, SP_UNSIGNED);
          table->field[7]->store(fsusage->free, SP_UNSIGNED);
          table->field[8]->store(fsusage->used, SP_UNSIGNED);
          table->field[9]->store(fsusage->avail, SP_UNSIGNED);
          table->field[10]->store(fsusage->files, SP_UNSIGNED);
          table->field[11]->store(fsusage->free_files, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_FILESYSTEM: Couldn't fetch fsusage data for "
                            "fslist %d.", i); 
        }
        
        error= error | schema_table_store_record(thd, table);   
        free(fsusage);
      }
  
      sigar_file_system_list_destroy(t, &fslist);
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_fs_plugin_init(void *p)
{
  DBUG_ENTER("init_fs_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_fs_schema;
  schema->fill_table= os_fs_fill_table;
  
  DBUG_RETURN(0);
}


static int os_fs_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_fs_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_fs_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_fs= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_fs_plugin,                                 
  "OS_FILESYSTEM",                                       
  SP_AUTHOR,                 
  "SIGAR Filesystem Details.",                           
  PLUGIN_LICENSE_GPL,                              
  os_fs_plugin_init,                             
  os_fs_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
