#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_info_schema[]=
{
  {"VARIABLE", 22,  MYSQL_TYPE_VARCHAR, 0, 0, "Variable"},
  {"VALUE",    256, MYSQL_TYPE_VARCHAR, 0, 0, "Value"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};

/* Fill row with data */
static int fill_row(const char *var_field, char *value_field, THD *thd, 
                    TABLE_LIST *tables)
{
  DBUG_ENTER("summary_plugin_fill_row");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table= tables->table;
  table->field[0]->store(var_field, strlen(var_field), scs);
  table->field[1]->store(value_field, strlen(value_field), scs);
  
  DBUG_RETURN(schema_table_store_record(thd, table));
}


int os_info_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("summary_plugin_fill_table");
  int error= 0;
  SP_RETURN_TYPE return_code;
  char *value_buf;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
  
    /* init sigar structures */
    sigar_sys_info_t *sysinfo;
    sigar_net_info_t *netinfo;
    sigar_net_stat_t *netstat;
    sigar_proc_stat_t *procstat;
    sigar_uptime_t *uptime;
    sigar_loadavg_t *loadavg;
    sysinfo= (sigar_sys_info_t*) malloc(sizeof(sigar_sys_info_t));
    netinfo= (sigar_net_info_t*) malloc(sizeof(sigar_net_info_t));
    netstat= (sigar_net_stat_t*) malloc(sizeof(sigar_net_stat_t));
    procstat= (sigar_proc_stat_t*) malloc(sizeof(sigar_proc_stat_t));
    uptime= (sigar_uptime_t*) malloc(sizeof(sigar_uptime_t));
    loadavg= (sigar_loadavg_t*) malloc(sizeof(sigar_loadavg_t));
  
    /* get OS information */
    return_code= sigar_sys_info_get(t, sysinfo);
    if (return_code == SIGAR_OK)
    {
      error= error | fill_row("NAME", sysinfo->name, thd, tables);
      error= error | fill_row("VERSION", sysinfo->version, thd, tables);
      error= error | fill_row("ARCHITECTURE", sysinfo->arch, thd, tables);
      error= error | fill_row("MACHINE", sysinfo->machine, thd, tables);
      error= error | fill_row("DESCRIPTION", sysinfo->description, thd, tables);
      error= error | fill_row("PATCH_LEVEL", sysinfo->patch_level, thd, tables);
      error= error | fill_row("VENDOR", sysinfo->vendor, thd, tables);
      error= error | fill_row("VENDOR_VERSION", sysinfo->vendor_version, thd, 
                              tables);
      error= error | fill_row("VENDOR_NAME", sysinfo->vendor_name, thd, tables);
      error= error | fill_row("VENDOR_CODE_NAME", sysinfo->vendor_code_name, 
                              thd, tables);
    }
    else
    {
      sql_print_warning("OS_SUMMARY: Couldn't fetch system info data.");
    }
    /* get network info */
    return_code= sigar_net_info_get(t, netinfo);
    if (return_code == SIGAR_OK)
    {
      error= error | fill_row("DEFAULT_GATEWAY", netinfo->default_gateway, thd, 
                              tables);
      error= error | fill_row("DOMAIN_NAME", netinfo->domain_name, thd, tables);
      error= error | fill_row("HOSTNAME", netinfo->host_name, thd, tables);
      error= error | fill_row("PRIMARY_DNS", netinfo->primary_dns, thd, tables);
      error= error | fill_row("SECONDARY_DNS", netinfo->secondary_dns, thd, 
                              tables);
    }
    else
    {
      sql_print_warning("OS_SUMMARY: Couldn't fetch network info data.");
    }
    /* get netstat info */
    return_code= sigar_net_stat_get(t, netstat, SIGAR_NETCONN_TCP | 
                                    SIGAR_NETCONN_UDP | SIGAR_NETCONN_RAW);
    if (return_code == SIGAR_OK)
    {
      asprintf(&value_buf, SP_UINT32, netstat->all_inbound_total);
      error= error | fill_row("TOTAL_INBOUND", value_buf, thd, tables);
      asprintf(&value_buf, SP_UINT32, netstat->all_outbound_total);
      error= error | fill_row("TOTAL_OUTBOUND", value_buf, thd, tables);
    }
    else
    {
      sql_print_warning("OS_SUMMARY: Couldn't fetch network stat data.");
    }
    /* get processes info */
    return_code= sigar_proc_stat_get(t, procstat);
    if (return_code == SIGAR_OK)
    {
      asprintf(&value_buf, SP_UINT64, procstat->total);
      error= error | fill_row("PROCESSES_TOTAL", value_buf, thd, tables);
      asprintf(&value_buf, SP_UINT64, procstat->sleeping);
      error= error | fill_row("PROCESSES_SLEEPING", value_buf, thd, tables);
      asprintf(&value_buf, SP_UINT64, procstat->running);
      error= error | fill_row("PROCESSES_RUNNING", value_buf, thd, tables);
      asprintf(&value_buf, SP_UINT64, procstat->zombie);
      error= error | fill_row("PROCESSES_ZOMBIE", value_buf, thd, tables);
      asprintf(&value_buf, SP_UINT64, procstat->stopped);
      error= error | fill_row("PROCESSES_STOPPED", value_buf, thd, tables);
      asprintf(&value_buf, SP_UINT64, procstat->idle);
      error= error | fill_row("PROCESSES_IDLE", value_buf, thd, tables);
      asprintf(&value_buf, SP_UINT64, procstat->threads);
      error= error | fill_row("THREADS", value_buf, thd, tables);
    }
    else
    {
      sql_print_warning("OS_SUMMARY: Couldn't fetch processes info data.");
    }
    /* get uptime */
    return_code= sigar_uptime_get(t, uptime);
    if (return_code == SIGAR_OK)
    {
      asprintf(&value_buf, SP_DOUBLE, uptime->uptime);
      error= error | fill_row("UPTIME", value_buf, thd, tables);
    }
    else
    {
      sql_print_warning("OS_SUMMARY: Couldn't fetch uptime data.");
    }
  
    /* get load average */
    return_code= sigar_loadavg_get(t, loadavg);
    if (return_code == SIGAR_OK)
    {
      asprintf(&value_buf, SP_DOUBLE, loadavg->loadavg[0]);
      error= error | fill_row("LOAD_AVERAGE (1 MIN)", value_buf, thd, tables);
      asprintf(&value_buf, SP_DOUBLE, loadavg->loadavg[1]);
      error= error | fill_row("LOAD_AVERAGE (5 MIN)", value_buf, thd, tables);
      asprintf(&value_buf, SP_DOUBLE, loadavg->loadavg[2]);
      error= error | fill_row("LOAD_AVERAGE (15 MIN)", value_buf, thd, tables);
    }
    else
    {
      sql_print_warning("OS_SUMMARY: Couldn't fetch load average data.");
    }
    /* free sigar structures */
    free(sysinfo);
    free(netinfo);
    free(netstat);
    free(procstat);
    free(uptime);
    free(loadavg);
  
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_info_plugin_init(void *p)
{
  DBUG_ENTER("init_summary_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_info_schema;
  schema->fill_table= os_info_fill_table;
  
  DBUG_RETURN(0);
}


static int os_info_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_summary_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_info_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_info=
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_info_plugin,                                 
  "OS_SUMMARY",                                       
  SP_AUTHOR,                 
  "SIGAR Operating System Summary.",                            
  PLUGIN_LICENSE_GPL,                              
  os_info_plugin_init,                             
  os_info_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL                                             
};
