#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_memswap_schema[]=
{
  {"VARIABLE", 16,                  MYSQL_TYPE_VARCHAR,  0, 0, "Variable"},
  {"VALUE",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
               "Value"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};

/* Fill row with plugin data */
static int fill_row(const char *var_field, sigar_uint64_t value, THD *thd, 
                    TABLE_LIST *tables)
{
  DBUG_ENTER("memory_plugin_fill_row");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table= tables->table;
  
  table->field[0]->store(var_field, strlen(var_field), scs);
  table->field[1]->store(value, SP_UNSIGNED);
  
  DBUG_RETURN(schema_table_store_record(thd, table));
}


int os_memswap_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("memory_plugin_fill_table");
  int error= 0;
  SP_RETURN_TYPE return_code;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* init sigar structures */
    sigar_mem_t *memory;
    sigar_swap_t *swap;
    memory= (sigar_mem_t*) malloc(sizeof(sigar_mem_t));
    swap= (sigar_swap_t*) malloc(sizeof(sigar_swap_t));
  
    /* get memory data */
    return_code= sigar_mem_get(t, memory);
    if (return_code == SIGAR_OK)
    {
      error= error | fill_row("MEMORY_RAM", memory->ram, thd, tables);
      error= error | fill_row("RAM_TOTAL", memory->total, thd, tables);
      error= error | fill_row("RAM_USED", memory->used, thd, tables);
      error= error | fill_row("RAM_FREE", memory->free, thd, tables);
      error= error | fill_row("RAM_ACTUAL_USED", memory->actual_used, thd, 
                              tables);
      error= error | fill_row("RAM_ACTUAL_FREE", memory->actual_free, thd, 
                              tables);
    }
    else
    {
      sql_print_warning("OS_MEMORY_RAM_SWAP: Couldn't fetch RAM memory data.");
    }
    /* get swap data */
    return_code= sigar_swap_get(t, swap);
    if (return_code == SIGAR_OK)
    {
      error= error | fill_row("SWAP_TOTAL", swap->total, thd, tables);
      error= error | fill_row("SWAP_USED", swap->used, thd, tables);
      error= error | fill_row("SWAP_FREE", swap->free, thd, tables);
      error= error | fill_row("SWAP_PAGES_IN", swap->page_in, thd, tables);
      error= error | fill_row("SWAP_PAGES_OUT", swap->page_out, thd, tables);
    }
    else
    {
      sql_print_warning("OS_MEMORY_RAM_SWAP: Couldn't fetch swap memory data.");
    }
    /* deinit sigar structures */
    free(memory);
    free(swap);
          
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_memswap_plugin_init(void *p)
{
  DBUG_ENTER("init_memory_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_memswap_schema;
  schema->fill_table= os_memswap_fill_table;
  
  DBUG_RETURN(0);
}


static int os_memswap_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_memory_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_memswap_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_memswap= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_memswap_plugin,                                 
  "OS_MEMORY_RAM_SWAP",                                       
  SP_AUTHOR,                 
  "SIGAR RAM && Swap Memory Information.",                           
  PLUGIN_LICENSE_GPL,                              
  os_memswap_plugin_init,                             
  os_memswap_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
