#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_nwif_schema[]=
{
  {"NAME",        16,                  MYSQL_TYPE_VARCHAR,  0, 0, "Name"},
  {"TYPE",        64,                  MYSQL_TYPE_VARCHAR,  0, 0, "Type"},
  {"DESCRIPTION", 256,                 MYSQL_TYPE_VARCHAR,  0, 0, "Desc"},
  {"MAC_ADDRESS", SP_MAC_ADDR_LENGTH,  MYSQL_TYPE_VARCHAR,  0, 0, "Mac_addr"},
  {"IP_ADDRESS",  SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Ip_addr"},
  {"DESTINATION", SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Dest"},
  {"BROADCAST",   SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Broadcast"},
  {"NETMASK",     SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Netmask"},
  {"FLAGS",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Flags"},
  {"MTU",         SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Mtu"},
  {"METRICS",     SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Metrics"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};

/*
  Transform internet address into human readable form. Address
  could be one of the following: MAC, IPv4 or undefined.
*/
static char* inet_addr_to_str(sigar_net_address_t *net_addr)
{
  char *A, *B, *C, *D;
  char *str_addr;
  sigar_uint32_t addr= net_addr->addr.in;
  unsigned char *mac=  &net_addr->addr.mac[0];
  
  switch (net_addr->family){
  case SIGAR_AF_INET:
    asprintf(&D, SP_UINT32, addr >> 24);
    asprintf(&C, SP_UINT32, (addr << 8) >> 24);
    asprintf(&B, SP_UINT32, (addr << 16) >> 24);
    asprintf(&A, SP_UINT32, (addr << 24) >> 24);
    asprintf(&str_addr, "%s.%s.%s.%s", A, B, C, D); 
    return str_addr;
  case SIGAR_AF_LINK:
    asprintf(&str_addr, "%02X:%02X:%02X:%02X:%02X:%02X", (mac[0] & 0xff), 
            (mac[1] & 0xff), (mac[2] & 0xff), (mac[3] & 0xff), (mac[4] & 0xff), 
            (mac[5] & 0xff)); 
    return str_addr;
  default:
    asprintf(&str_addr, "%s", "UNDEFINED");
    return str_addr;
  }
}


int os_nwif_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("nwif_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  char *inet_address;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each nw interface */
    sigar_net_interface_list_t iflist;
    return_code= sigar_net_interface_list_get(t, &iflist);
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < iflist.number; i++) 
      {
        sigar_net_interface_config_t *ifconfig;
        ifconfig= (sigar_net_interface_config_t*) 
                   malloc(sizeof(sigar_net_interface_config_t));
        
        return_code= sigar_net_interface_config_get(t, iflist.data[i], 
                                                    ifconfig);
        //inet_address= (char*) malloc(SIGAR_INET6_ADDRSTRLEN);
        if (return_code == SIGAR_OK)
        {
          table->field[0]->store(ifconfig->name, strlen(ifconfig->name), scs);
          table->field[1]->store(ifconfig->type, strlen(ifconfig->type), scs);
          table->field[2]->store(ifconfig->description, 
                                 strlen(ifconfig->description), scs);
          /* convert mac && ip addresses to string */
          inet_address= inet_addr_to_str(&ifconfig->hwaddr);
          table->field[3]->store(inet_address, strlen(inet_address), scs);
          
          inet_address= inet_addr_to_str(&ifconfig->address);
          table->field[4]->store(inet_address, strlen(inet_address), scs);
          
          inet_address= inet_addr_to_str(&ifconfig->destination);
          table->field[5]->store(inet_address, strlen(inet_address), scs);
          
          inet_address= inet_addr_to_str(&ifconfig->broadcast);
          table->field[6]->store(inet_address, strlen(inet_address), scs);
          
          inet_address= inet_addr_to_str(&ifconfig->netmask);
          table->field[7]->store(inet_address, strlen(inet_address), scs);
          
          table->field[8]->store(ifconfig->flags, SP_UNSIGNED);
          table->field[9]->store(ifconfig->mtu, SP_UNSIGNED);
          table->field[10]->store(ifconfig->metric, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_NETWORK_INTERFACES: Couldn't fetch data for %s",
                            iflist.data[i]);    
        }
        
        error= error | schema_table_store_record(thd, table);
        free(ifconfig);
      }
      sigar_net_interface_list_destroy(t, &iflist);
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_nwif_plugin_init(void *p)
{
  DBUG_ENTER("init_nwif_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_nwif_schema;
  schema->fill_table= os_nwif_fill_table;
  
  DBUG_RETURN(0);
}


static int os_nwif_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_nwif_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_nwif_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_nwif= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_nwif_plugin,                                 
  "OS_NETWORK_INTERFACES",                                       
  SP_AUTHOR,                 
  "SIGAR Network Interfaces.",                           
  PLUGIN_LICENSE_GPL,                              
  os_nwif_plugin_init,                             
  os_nwif_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
