#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_nwif_cons_schema[]=
{
  {"LOCAL_PORT",     SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                     MY_I_S_UNSIGNED, "Lport"},
  {"LOCAL_ADDRESS",  SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Laddr"},
  {"REMOTE_PORT",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                     MY_I_S_UNSIGNED, "Rport"},
  {"REMOTE_ADDRESS", SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Raddr"},
  {"UID",            SP_UID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Uid"},
  {"INODE",          SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                     MY_I_S_UNSIGNED, "Inode"},
  {"TYPE",           SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, 0, "Type"},
  {"STATE",          SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, 0, "State"},
  {"SEND_QUEUE",     SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                     MY_I_S_UNSIGNED, "Squeue"},
  {"RECEIVE_QUEUE",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                     MY_I_S_UNSIGNED, "Rqueue"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};

/* Transform inet address into IPv4 format */
static char* inet_addr_to_str(sigar_net_address_t *net_addr)
{
  char *A, *B, *C, *D;
  char *str_addr;
  sigar_uint32_t addr= net_addr->addr.in;
  
  asprintf(&D, SP_UINT32, addr >> 24);
  asprintf(&C, SP_UINT32, (addr << 8) >> 24);
  asprintf(&B, SP_UINT32, (addr << 16) >> 24);
  asprintf(&A, SP_UINT32, (addr << 24) >> 24);
  asprintf(&str_addr, "%s.%s.%s.%s", A, B, C, D); 
  return str_addr;
}


int os_nwif_cons_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("nwif_con_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  char *inet_address;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  { 
    /* get data on each connection */
    sigar_net_connection_list_t netcons;
    /* get all types of connections */
    return_code= sigar_net_connection_list_get(t, &netcons, 
                                               SIGAR_NETCONN_CLIENT | 
                                               SIGAR_NETCONN_SERVER | 
                                               SIGAR_NETCONN_TCP | 
                                               SIGAR_NETCONN_UDP | 
                                               SIGAR_NETCONN_RAW |
                                               SIGAR_NETCONN_UNIX);
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < netcons.number; i++) 
      {
        sigar_net_connection_t *netcon= &netcons.data[i];

        if (netcon != NULL)
        {
          table->field[0]->store(netcon->local_port, SP_UNSIGNED);
          inet_address= inet_addr_to_str(&netcon->local_address);
          table->field[1]->store(inet_address, strlen(inet_address), scs);
          table->field[2]->store(netcon->remote_port, SP_UNSIGNED);
          inet_address= inet_addr_to_str(&netcon->remote_address);
          table->field[3]->store(inet_address, strlen(inet_address), scs);
          table->field[4]->store(netcon->uid, SP_UID_UNSIGNED);
          table->field[5]->store(netcon->inode, SP_UNSIGNED);
          table->field[6]->store(netcon->type, SP_SIGNED);
          table->field[7]->store(netcon->state, SP_SIGNED);  
          table->field[8]->store(netcon->send_queue, SP_UNSIGNED);
          table->field[9]->store(netcon->receive_queue, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_NETWORK_CONNECTIONS: Couldn't fetch network "
                            "connection data for connection %d", i);  
        }
        
        error= error | schema_table_store_record(thd, table);
      }
      sigar_net_connection_list_destroy(t, &netcons);
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_nwif_cons_plugin_init(void *p)
{
  DBUG_ENTER("init_nwif_con_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_nwif_cons_schema;
  schema->fill_table= os_nwif_cons_fill_table;
  
  DBUG_RETURN(0);
}


static int os_nwif_cons_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_nwif_con_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_nwif_cons_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_nwif_cons= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_nwif_cons_plugin,                                 
  "OS_NETWORK_CONNECTIONS",                                       
  SP_AUTHOR,                 
  "SIGAR Network Connections Details.",                           
  PLUGIN_LICENSE_GPL,                              
  os_nwif_cons_plugin_init,                             
  os_nwif_cons_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
