#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_nwif_routes_schema[]=
{
  {"DESTINATION", SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Dest"},
  {"GATEWAY",     SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Gateway"},
  {"NETMASK",     SP_INET_ADDR_LENGTH, MYSQL_TYPE_VARCHAR,  0, 0, "Netmask"},
  {"FLAGS",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Flags"},
  {"REFCNT",      SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Refcnt"},
  {"USE",         SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Use"},
  {"METRIC",      SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Metric"},
  {"MTU",         SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Mtu"},
  {"WINDOW",      SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "window"},
  {"IRTT",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                  "Irtt"},
  {"INTERFACE",   16,                  MYSQL_TYPE_VARCHAR,  0, 0, "If_name"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};

/* Transform inet address into IPv4 */
static char* inet_addr_to_str(sigar_net_address_t *net_addr)
{
  char *A, *B, *C, *D;
  char *str_addr;
  sigar_uint32_t addr= net_addr->addr.in;
  
  asprintf(&D, SP_UINT32, addr >> 24);
  asprintf(&C, SP_UINT32, (addr << 8) >> 24);
  asprintf(&B, SP_UINT32, (addr << 16) >> 24);
  asprintf(&A, SP_UINT32, (addr << 24) >> 24);
  asprintf(&str_addr, "%s.%s.%s.%s", A, B, C, D); 
  return str_addr;
}


int os_nwif_routes_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("nwif_routes_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  char *inet_address;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each nw interface */
    sigar_net_route_list_t routes;
    return_code= sigar_net_route_list_get(t, &routes);
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < routes.number; i++) 
      {
        sigar_net_route_t *route= &routes.data[i];
        
        if (route != NULL)
        {
          inet_address= inet_addr_to_str(&route->destination);
          table->field[0]->store(inet_address, strlen(inet_address), scs);
          inet_address= inet_addr_to_str(&route->gateway);
          table->field[1]->store(inet_address, strlen(inet_address), scs);
          inet_address= inet_addr_to_str(&route->mask);
          table->field[2]->store(inet_address, strlen(inet_address), scs);
        
          table->field[3]->store(route->flags, SP_UNSIGNED);
          table->field[4]->store(route->refcnt, SP_UNSIGNED);
          table->field[5]->store(route->use, SP_UNSIGNED);
          table->field[6]->store(route->metric, SP_UNSIGNED);
          table->field[7]->store(route->mtu, SP_UNSIGNED);  
          table->field[8]->store(route->window, SP_UNSIGNED);
          table->field[9]->store(route->irtt, SP_UNSIGNED);
          table->field[10]->store(route->ifname, strlen(route->ifname), scs);
        }
        else
        {
          sql_print_warning("OS_NETWORK_ROUTES: Couldn't fetch network routes"
                            " data for route %d", i);
        }
        error= error | schema_table_store_record(thd, table);
      }
      sigar_net_route_list_destroy(t, &routes);
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_nwif_routes_plugin_init(void *p)
{
  DBUG_ENTER("init_nwif_routes_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_nwif_routes_schema;
  schema->fill_table= os_nwif_routes_fill_table;
  DBUG_RETURN(0);
}


static int os_nwif_routes_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_nwif_routes_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_nwif_routes_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_nwif_routes= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_nwif_routes_plugin,                                 
  "OS_NETWORK_ROUTES",                                       
  SP_AUTHOR,                 
  "SIGAR Network Routes Information.",                           
  PLUGIN_LICENSE_GPL,                              
  os_nwif_routes_plugin_init,                             
  os_nwif_routes_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
