#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_nwif_stat_schema[]=
{
  {"INTERFACE",     16,                  MYSQL_TYPE_VARCHAR,  0, 0, "If"},
  {"RX_PACKETS",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Rx_pcts"},
  {"RX_BYTES",      SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Rx_bytes"},
  {"RX_ERRORS",     SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Rx_err"},
  {"RX_DROPPED",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Rx_drop"},
  {"RX_OVERRUNS",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Rx_ovrn"},
  {"RX_FRAME",      SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Rx_frame"},
  {"TX_PACKETS",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Tx_pcts"},
  {"TX_BYTES",      SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Tx_bytes"},
  {"TX_ERRORS",     SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Tx_err"},
  {"TX_DROPPED",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Tx_drop"},
  {"TX_OVERRUNS",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Tx_ovrn"},
  {"TX_COLLISIONS", SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Tx_clns"},
  {"TX_CARRIER",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Tx_car"},
  {"SPEED",         SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                    MY_I_S_UNSIGNED, "Speed"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};


int os_nwif_stat_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("nwif_stat_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each nw interface */
    sigar_net_interface_list_t iflist;
    return_code= sigar_net_interface_list_get(t, &iflist);
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < iflist.number; i++) 
      {
        sigar_net_interface_stat_t *netstat;
        netstat= (sigar_net_interface_stat_t*)
                 malloc(sizeof(sigar_net_interface_stat_t));
        
        table->field[0]->store(iflist.data[i], strlen(iflist.data[i]), scs);
        return_code= sigar_net_interface_stat_get(t, iflist.data[i], netstat);
        if (return_code == SIGAR_OK)
        {
          table->field[1]->store(netstat->rx_packets, SP_UNSIGNED);
          table->field[2]->store(netstat->rx_bytes, SP_UNSIGNED);
          table->field[3]->store(netstat->rx_errors, SP_UNSIGNED);
          table->field[4]->store(netstat->rx_dropped, SP_UNSIGNED);
          table->field[5]->store(netstat->rx_overruns, SP_UNSIGNED);
          table->field[6]->store(netstat->rx_frame, SP_UNSIGNED);
          table->field[7]->store(netstat->tx_packets, SP_UNSIGNED);
          table->field[8]->store(netstat->tx_bytes, SP_UNSIGNED);
          table->field[9]->store(netstat->tx_errors, SP_UNSIGNED);
          table->field[10]->store(netstat->tx_dropped, SP_UNSIGNED);
          table->field[11]->store(netstat->tx_overruns, SP_UNSIGNED);
          table->field[12]->store(netstat->tx_collisions, SP_UNSIGNED);
          table->field[13]->store(netstat->tx_carrier, SP_UNSIGNED);
          table->field[14]->store(netstat->speed, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_NETWORK_INTERFACES_STAT: Couldn't fetch data "
                            "for interface %s", iflist.data[i]);
        }
        
        error= error | schema_table_store_record(thd, table);
        free(netstat);
      }
      sigar_net_interface_list_destroy(t, &iflist);
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_nwif_stat_plugin_init(void *p)
{
  DBUG_ENTER("init_nwif_stat_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_nwif_stat_schema;
  schema->fill_table= os_nwif_stat_fill_table;
  DBUG_RETURN(0);
}


static int os_nwif_stat_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_nwif_stat_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_nwif_stat_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_nwif_stat= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_nwif_stat_plugin,                                 
  "OS_NETWORK_INTERFACES_STAT",                                       
  SP_AUTHOR,                 
  "SIGAR Network Interfaces' Statistics.",                           
  PLUGIN_LICENSE_GPL,                              
  os_nwif_stat_plugin_init,                             
  os_nwif_stat_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
