#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_ps_schema[]=
{
  {"PID",      SP_PID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Pid"},
  {"UID",      SP_UID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Uid"},
  {"GID",      SP_GID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Gid"},
  {"EUID",     SP_UID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Euid"},
  {"EGID",     SP_GID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Egid"},
  {"USER",     SIGAR_CRED_NAME_MAX, MYSQL_TYPE_VARCHAR,  0, 0, "User"},
  {"GROUP",    SIGAR_CRED_NAME_MAX, MYSQL_TYPE_VARCHAR,  0, 0, "Group"},
  {"PS_NAME",  SIGAR_PROC_NAME_LEN, MYSQL_TYPE_VARCHAR,  0, 0, "Ps_name"},
  {"STATE",    6,                   MYSQL_TYPE_VARCHAR,  0, 0, "State"},
  {"PPID",     SP_PID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Ppid"},
  {"TTY",      SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, 0, "tty"},
  {"PRIORITY", SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, 0, "Priority"},
  {"NICE",     SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, 0, "Nice"},
  {"CPU",      SP_ANY_INT32_LENGTH, MYSQL_TYPE_LONG,     0, 0, "Cpu"},
  {"THREADS",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
               "Threads"},
  {"FD",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
               "Fd"},
  {"EXE_NAME", SIGAR_PATH_MAX+1,    MYSQL_TYPE_VARCHAR,  0, 0, "Exe_name"},
  {"CWD",      SIGAR_PATH_MAX+1,    MYSQL_TYPE_VARCHAR,  0, 0, "Cwd"},
  {"ROOT",     SIGAR_PATH_MAX+1,    MYSQL_TYPE_VARCHAR,  0, 0, "Root"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};


int os_ps_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("ps_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each user */
    sigar_proc_list_t proclist;
    return_code= sigar_proc_list_get(t, &proclist);
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < proclist.number; i++) 
      {
        /* init sigar structures */
        sigar_proc_cred_t      *proccred;
        sigar_proc_cred_name_t *proccredname;
        sigar_proc_state_t     *procstate;
        sigar_proc_fd_t        *procfd;
        sigar_proc_exe_t       *procexe;
        proccred= (sigar_proc_cred_t*) malloc(sizeof(sigar_proc_cred_t));
        proccredname= (sigar_proc_cred_name_t*) 
                       malloc(sizeof(sigar_proc_cred_name_t));
        procstate= (sigar_proc_state_t*) malloc(sizeof(sigar_proc_state_t));
        procfd= (sigar_proc_fd_t*) malloc(sizeof(sigar_proc_fd_t));
        procexe= (sigar_proc_exe_t*) malloc(sizeof(sigar_proc_exe_t));
        
        /* get proc credits, credit names, state, filed desc, exe names && dirs */ 
        table->field[0]->store(proclist.data[i], SP_PID_UNSIGNED);
        
        return_code= sigar_proc_cred_get(t, proclist.data[i], proccred);  
        if (return_code == SIGAR_OK)
        {
          table->field[1]->store(proccred->uid, SP_UID_UNSIGNED);
          table->field[2]->store(proccred->gid, SP_GID_UNSIGNED);
          table->field[3]->store(proccred->euid, SP_UID_UNSIGNED);
          table->field[4]->store(proccred->egid, SP_GID_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_PROCESSES: Couldn't fetch proc credit data for "
                            "process %d", proclist.data[i]);  
        }
        
        return_code= sigar_proc_cred_name_get(t, proclist.data[i], proccredname);
        if (return_code == SIGAR_OK)
        {
          table->field[5]->store(proccredname->user, strlen(proccredname->user), 
                                 scs);
          table->field[6]->store(proccredname->group, strlen(proccredname->group), 
                                 scs);
        }
        else
        {
          sql_print_warning("OS_PROCESSES: Couldn't fetch proc credit name data"
                            " for process %d", proclist.data[i]);  
        }
        
        return_code= sigar_proc_state_get(t, proclist.data[i], procstate);
        if (return_code == SIGAR_OK)
        {
          table->field[7]->store(procstate->name, strlen(procstate->name), scs);
          switch (procstate->state){
          case 'S': 
            table->field[8]->store("SLEEP", strlen("SLEEP"), scs);
            break;
          case 'R': 
            table->field[8]->store("RUN", strlen("RUN"), scs);
            break;
          case 'T': 
            table->field[8]->store("STOP", strlen("STOP"), scs);
            break;
          case 'Z': 
            table->field[8]->store("ZOMBIE", strlen("ZOMBIE"), scs);
            break;
          case 'D': 
            table->field[8]->store("IDLE", strlen("IDLE"), scs);
            break;
          default: 
            table->field[8]->store("N/A", strlen("N/A"), scs);
          }
          table->field[9]->store(procstate->ppid, SP_PID_UNSIGNED);
          table->field[10]->store(procstate->tty, SP_SIGNED);
          table->field[11]->store(procstate->priority, SP_SIGNED);
          table->field[12]->store(procstate->nice, SP_SIGNED);
          table->field[13]->store(procstate->processor, SP_SIGNED);
          table->field[14]->store(procstate->threads, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_PROCESSES: Couldn't fetch proc state data for "
                            "process %d", proclist.data[i]);  
        }
        
        return_code= sigar_proc_fd_get(t, proclist.data[i], procfd);    
        if (return_code == SIGAR_OK)
        {
          table->field[15]->store(procfd->total, SP_UNSIGNED); 
        }
        else
        {
          sql_print_warning("OS_PROCESSES: Couldn't fetch proc file descriptor "
                            "data for process %d", proclist.data[i]);  
        }
        
        return_code= sigar_proc_exe_get(t, proclist.data[i], procexe);
        if (return_code == SIGAR_OK)
        {
          table->field[16]->store(procexe->name, strlen(procexe->name), scs);
          table->field[17]->store(procexe->cwd, strlen(procexe->cwd), scs);
          table->field[18]->store(procexe->root, strlen(procexe->root), scs);  
        }
        else
        {
          sql_print_warning("OS_PROCESSES: Couldn't fetch proc exe data for "
                            "process %d", proclist.data[i]);  
        }
        
        error= error | schema_table_store_record(thd, table);
    
        /* free sigar structs */
        free(proccred);
        free(proccredname);
        free(procstate);
        free(procfd);
        free(procexe);
      }
      sigar_proc_list_destroy(t, &proclist);
  
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_ps_plugin_init(void *p)
{
  DBUG_ENTER("init_ps_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_ps_schema;
  schema->fill_table= os_ps_fill_table;
  DBUG_RETURN(0);
}


static int os_ps_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_ps_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_ps_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_ps= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_ps_plugin,                                 
  "OS_PROCESSES",                                       
  SP_AUTHOR,                 
  "SIGAR Operating System Processes.",                           
  PLUGIN_LICENSE_GPL,                              
  os_ps_plugin_init,                             
  os_ps_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
