#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_ps_stat_schema[]=
{
  {"PID",             SP_PID_LENGTH,       ID_FIELD_TYPE,       0, 0, "Pid"},
  {"MEM_SIZE",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "size"},
  {"MEM_RESIDENT",    SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "rsdt"},
  {"MEM_SHARE",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "share"},
  {"MEM_MIN_FAULTS",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "minflt"},
  {"MEM_MAJ_FAULTS",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "majflt"},
  {"MEM_PAGE_FAULTS", SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "pflts"},
  {"CPU_START_TIME",  SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "start"},
  {"CPU_USER",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "user"},
  {"CPU_SYS",         SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "sys"},
  {"CPU_TOTAL",       SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "total"},
  {"CPU_LAST_TIME",   SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                      MY_I_S_UNSIGNED, "last"},
  {"CPU_PERCENT",     20,                  MYSQL_TYPE_DOUBLE,   0, 0, "pcnt"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};


int os_ps_stat_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("ps_stat_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each user */
    sigar_proc_list_t proclist;
    return_code= sigar_proc_list_get(t, &proclist);
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < proclist.number; i++) 
      {
        /* init sigar structs */ 
        sigar_proc_mem_t *procmem;
        sigar_proc_cpu_t *proccpu;
        procmem= (sigar_proc_mem_t*) malloc(sizeof(sigar_proc_mem_t));
        proccpu= (sigar_proc_cpu_t*) malloc(sizeof(sigar_proc_cpu_t));
        
        /* get memory && cpu usage */
        return_code= sigar_proc_mem_get(t, proclist.data[i], procmem);
        if (return_code == SIGAR_OK)
        {
          table->field[0]->store(proclist.data[i], SP_PID_UNSIGNED);
          table->field[1]->store(procmem->size, SP_UNSIGNED);
          table->field[2]->store(procmem->resident, SP_UNSIGNED);
          table->field[3]->store(procmem->share, SP_UNSIGNED);
          table->field[4]->store(procmem->minor_faults, SP_UNSIGNED);
          table->field[5]->store(procmem->major_faults, SP_UNSIGNED);
          table->field[6]->store(procmem->page_faults, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_PROCESSES_STAT: Couldn't fetch memory data for "
                            "process %d", proclist.data[i]);  
        }
        
        return_code= sigar_proc_cpu_get(t, proclist.data[i], proccpu);
        if (return_code == SIGAR_OK)
        {
          table->field[7]->store(proccpu->start_time, SP_UNSIGNED);
          table->field[8]->store(proccpu->user, SP_UNSIGNED);
          table->field[9]->store(proccpu->sys, SP_UNSIGNED);
          table->field[10]->store(proccpu->total, SP_UNSIGNED);
          table->field[11]->store(proccpu->last_time, SP_UNSIGNED);
          table->field[12]->store(proccpu->percent);
        }
        else
        {
          sql_print_warning("OS_PROCESSES_STAT: Couldn't fetch cpu data for "
                            "process %d", proclist.data[i]);  
        }
        
        error= error | schema_table_store_record(thd, table);
        free(procmem);
        free(proccpu);
      }
      sigar_proc_list_destroy(t, &proclist);
    
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_ps_stat_plugin_init(void *p)
{
  DBUG_ENTER("init_ps_stat_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info=     os_ps_stat_schema;
  schema->fill_table=      os_ps_stat_fill_table;
  DBUG_RETURN(0);
}


static int os_ps_stat_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_ps_stat_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_ps_stat_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_ps_stat= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_ps_stat_plugin,                                 
  "OS_PROCESSES_STAT",                                       
  SP_AUTHOR,                 
  "SIGAR Processes Memory && CPU Usage Details.",                           
  PLUGIN_LICENSE_GPL,                              
  os_ps_stat_plugin_init,                             
  os_ps_stat_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
