#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_rlimits_schema[]=
{
  {"RESOURCE_LIMIT", 15,                  MYSQL_TYPE_VARCHAR,  0, 0, "Rlimit"},
  {"CURRENT",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                     MY_I_S_UNSIGNED, "Current"},
  {"MAXIMUM",        SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, 
                     MY_I_S_UNSIGNED, "Maximum"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};

/* Fill row */
static int fill_row(const char *var_field, sigar_uint64_t cur_field, 
                    sigar_uint64_t max_field, THD *thd, TABLE_LIST *tables)
{
  DBUG_ENTER("rlimits_plugin_fill_row");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table= tables->table;
  table->field[0]->store(var_field, strlen(var_field), scs);
  table->field[1]->store(cur_field, SP_UNSIGNED);
  table->field[2]->store(max_field, SP_UNSIGNED);
  DBUG_RETURN(schema_table_store_record(thd, table));
}


int os_rlimits_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("rlimits_plugin_fill_table");
  int error= 0;
  SP_RETURN_TYPE return_code;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get resource limits information */
    sigar_resource_limit_t *rlimit;
    rlimit= (sigar_resource_limit_t*) malloc(sizeof(sigar_resource_limit_t));
    
    return_code= sigar_resource_limit_get(t, rlimit);
    if (return_code == SIGAR_OK)
    {
      error= error | fill_row("CPU", rlimit->cpu_cur, rlimit->cpu_max, thd, 
                              tables);
      error= error | fill_row("FILESIZE", rlimit->file_size_cur, 
                              rlimit->file_size_max, thd, tables);
      error= error | fill_row("PIPE_BUFFER", rlimit->pipe_size_cur, 
                              rlimit->pipe_size_max, thd, tables);
      error= error | fill_row("DATA", rlimit->data_cur, rlimit->data_max, thd, 
                              tables);
      error= error | fill_row("STACK", rlimit->stack_cur, rlimit->stack_max, 
                              thd, tables);
      error= error | fill_row("CORE", rlimit->core_cur, rlimit->core_max, thd, 
                              tables);
      error= error | fill_row("MEMORY", rlimit->memory_cur, rlimit->memory_max, 
                              thd, tables);
      error= error | fill_row("PROCESSES", rlimit->processes_cur, 
                              rlimit->processes_max, thd, tables);
      error= error | fill_row("OPEN_FILES", rlimit->open_files_cur, 
                              rlimit->open_files_max, thd, tables);
      error= error | fill_row("VIRTUAL_MEMORY", rlimit->virtual_memory_cur, 
                              rlimit->virtual_memory_max, thd, tables);
  
      free(rlimit);
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
  DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_rlimits_plugin_init(void *p)
{
  DBUG_ENTER("init_rlimits_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_rlimits_schema;
  schema->fill_table= os_rlimits_fill_table;
  DBUG_RETURN(0);
}


static int os_rlimits_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_rlimits_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_rlimits_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_rlimits=
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_rlimits_plugin,                                 
  "OS_RESOURCE_LIMITS",                                       
  SP_AUTHOR,                 
  "SIGAR Operating System Resource Limits.",                            
  PLUGIN_LICENSE_GPL,                              
  os_rlimits_plugin_init,                             
  os_rlimits_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL                                             
};
