#include "splugin.h"

using namespace std;

/*
  Table data structures and fill functions
*/

static ST_FIELD_INFO os_users_schema[]=
{
  {"USER",      32,                  MYSQL_TYPE_VARCHAR,  0, 0, "User"},
  {"DEVICE",    32,                  MYSQL_TYPE_VARCHAR,  0, 0, "Device"},
  {"HOST",      256,                 MYSQL_TYPE_VARCHAR,  0, 0, "Host"},
  {"TIMESTAMP", SP_ANY_INT64_LENGTH, MYSQL_TYPE_LONGLONG, 0, MY_I_S_UNSIGNED, 
                "Time"},
  {0, 0, MYSQL_TYPE_NULL, 0, 0, 0}
};


int os_users_fill_table(THD *thd, TABLE_LIST *tables, COND *cond)
{
  DBUG_ENTER("users_plugin_fill_table");
  CHARSET_INFO *scs= system_charset_info;
  TABLE *table=      tables->table;
  int error=         0;
  SP_RETURN_TYPE return_code;
  size_t i;
  
  /* init sigar */
  sigar_t *t;
  return_code= sigar_open(&t);
  
  if (return_code == SIGAR_OK)
  {
    /* get data on each user */
    sigar_who_list_t wholist;
    return_code= sigar_who_list_get(t, &wholist);
    if (return_code == SIGAR_OK)
    {
      for (i = 0; i < wholist.number; i++) 
      {
        sigar_who_t *who= &wholist.data[i];
        if (who != NULL)
        {
          table->field[0]->store(who->user, strlen(who->user), scs);
          table->field[1]->store(who->device, strlen(who->device), scs);
          table->field[2]->store(who->host, strlen(who->host), scs);
          table->field[3]->store(who->time, SP_UNSIGNED);
        }
        else
        {
          sql_print_warning("OS_USERS: Couldn't fetch user data for "
                            "user %d", i);  
        }
        error= error | schema_table_store_record(thd, table);
      }
      sigar_who_list_destroy(t, &wholist);
    }
    else
    {
      my_error(ER_SP_FETCH_NO_DATA, MYF(0));
    }
    /* deinit sigar */
    sigar_close(t);
  }
  else
  {
    my_error(ER_SP_FETCH_NO_DATA, MYF(0));
  }
   DBUG_RETURN(error);
}

/*
  Plugin init and deinit functions 
*/

static int os_users_plugin_init(void *p)
{
  DBUG_ENTER("init_users_plugin");
  ST_SCHEMA_TABLE *schema= (ST_SCHEMA_TABLE *)p;
  schema->fields_info= os_users_schema;
  schema->fill_table= os_users_fill_table;
  DBUG_RETURN(0);
}


static int os_users_plugin_deinit(void *p)
{
  DBUG_ENTER("deinit_users_plugin");
  DBUG_RETURN(0);
}


struct st_mysql_information_schema os_users_plugin=
{ MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION };


struct st_mysql_plugin os_users= 
{
  MYSQL_INFORMATION_SCHEMA_PLUGIN,                 
  &os_users_plugin,                                 
  "OS_USERS",                                       
  SP_AUTHOR,                 
  "SIGAR Operating System User Details.",                           
  PLUGIN_LICENSE_GPL,                              
  os_users_plugin_init,                             
  os_users_plugin_deinit,                           
  SP_MAJOR_VERSION | SP_MINOR_VERSION,                                          
  NULL,                                            
  NULL,                                            
  NULL         
};
